
//-----------------------------------------------------------
//
//	Softbolts Ltd. (c) all rights reserved
//
//	Warning: This  program is  protected by copyright law 
//		 and   international  treaties.  Unauthorised
//		 reproduction or distribution of this program,
//		 or  any portion  or it, may result in severe
//		 civil  and  criminal  penalties, and will be
//		 prosecuted to  the  maximum extent possible.
//
//              Author: A. E. Dawson, Aug 1999
//              mailto: anthony@softbolts.com
//
//		Last Revised: 3 July 2002
//
//-----------------------------------------------------------

package com.softbolts.connectors.TBDB;

import com.softbolts.lib.*;

import java.io.*;
import java.util.*;
import java.lang.*;
import java.sql.*;


public class DBObject extends ServletObj implements Serializable
{
 Servlet svr = null;
 String table = null;
 String hostName = null;
 String setSqlStatement = null;

        //-----------------------------------------------------------
        DBObject(Servlet svr)
        {
                this.svr = svr;
		try {
			hostName = svr.getHostName();
			table = svr.getTable();
			setSqlStatement = svr.getSetSql();
		} catch (ThrowableString ex) {
			System.out.println("TBDB.DBObject() - " + ex.getMessage());
		}
        }

	//-----------------------------------------------------------
	public int run(NameValueTyp params)
	{
	 String method = null;

		if((method = (String) params.get("sysMethod")) == null) return -1;
		if (method.equals("Get")) return Get(params);
		if (method.equals("Set")) return Set(params);
		return 0;
	}

	//-----------------------------------------------------------
	public int Set(NameValueTyp params)
	{
		try {
			if (setSqlStatement.equals("") == false)
				setUsingSqlStatement(params);
			else
				setUsingTable(params);
		} catch (ThrowableString ex) {
		}

		return 0;
	}

	//-----------------------------------------------------------
	public void setUsingSqlStatement(NameValueTyp params) throws ThrowableString
	{
	 String statement = "";

		if (setSqlStatement.indexOf(hostName + ":") != -1)
		{
		 String tmpStr = setSqlStatement;
		 String key = null;
		 int p1 = 0;
		 int p2 = 0;

			System.out.println("SQL key substitution");
			while ((p2 = setSqlStatement.indexOf(hostName + ":", p1)) != -1)
			{
				statement += setSqlStatement.substring(p1, p2);
				p1 = setSqlStatement.indexOf("'", p2);
				key = setSqlStatement.substring(p2, p1);
				System.out.println("Got key: " + key + " for substitution");
				if (params.containsKey(key) == false)
				{
					System.out.println("Couldn't find key: " + key + 
							" in params for substitution setting to blank string");
					statement += "";
				}
				else
					statement += params.get(key);
			}
		}	
		else  statement = setSqlStatement;

		System.out.println("USING statement: " + statement);

                try {
                        svr.stmt.executeQuery(statement);
                } catch (SQLException ex) {
                        throw new ThrowableString("TBDB.Servlet::setUsingSqlStatement) - SQLException: " +
                                        ex.getMessage() + " SQL Statement was: " + setSqlStatement);
                }
	}

	//-----------------------------------------------------------
	public void setUsingTable(NameValueTyp params) throws ThrowableString
	{
	 TTableDescriptions td = svr.di.td;
	 String tableName = null;
	 String columnName = null;
	 TTable tab = null;
	 TColumn col = null;
	 String currentKey = null;
	 String tmpStatement = "insert or update into " + table + "(";
	 String statement = null;

		tab = td.get(tableName);
		Enumeration ce = tab.keys();
		while (ce.hasMoreElements())
		{
			columnName = (String) ce.nextElement();
			tmpStatement += columnName;
		}
		tmpStatement += ") values ('";

		ce = tab.keys();
		while (ce.hasMoreElements())
		{
			currentKey = hostName + ":" + table + "." + columnName;
	System.out.println("Searching for key: " + currentKey);

			if (params.containsKey(currentKey))
			{
				System.out.println("Found key: " + currentKey);
				tmpStatement += params.get(currentKey) + "',";
			}
			else
				tmpStatement += "',";
		}
		statement = tmpStatement.substring(0, tmpStatement.lastIndexOf(",")) + ")";

	System.out.println("About to execute statement: " + statement);
                try {
                        svr.stmt.executeQuery(statement);
                } catch (SQLException ex) {
                        throw new ThrowableString("TBDB.Servlet::setUsingTable) - SQLException: " +
                                        ex.getMessage() + " SQL Statement was: " + setSqlStatement);
                }
	}

	/**
	* Gets a database table contents by running an SQL statement.
	* The statement run is one of the following:
	* 1) That configured in the database connector property.
	* 2) Or if that is blank then the statement described by the SQL_STATEMENT paramater.
	* 3) Or if that is not present that created by "select * from " + the table specified in the table property value
	* 4) Or if that is not present that created by "select * from " + the sysObject parameter.
	* if none of the above are possible the method simply returns without doing anything.
	*/
	//-----------------------------------------------------------
	public int Get(NameValueTyp params)
	{
	 String sqlStatement = "";
	 ResultSet rs = null;
	 ResultSetMetaData rsmd = null;
	 String sqlId = "";
	 String val = "";
	 int numbOfCols = 0;

		if (!svr.configParams.get(svr.keySql).value.equals(""))
			sqlStatement = svr.configParams.get(svr.keySql).value;
		else if (params.containsKey("SQL_STATEMENT"))
			sqlStatement = params.get("SQL_STATEMENT");
		else if (svr.configParams.get(svr.keyTable).value != "")
			sqlStatement = "select * from " + svr.configParams.get(svr.keyTable).value;
		else 
			sqlStatement = "select * from " + params.get("sysObject");

		String colName = null;
		String preFix = svr.servletHostName + ":" + svr.assignedName + "." + params.get("sysObject") + ".";
		try {
			// System.out.println("USING SQL statement: " + sqlStatement);
                        rs = svr.stmt.executeQuery(sqlStatement);
			rsmd = rs.getMetaData();
			numbOfCols = rsmd.getColumnCount();
			if (params.containsKey("SQL_ID"))
				sqlId = params.get("SQL_ID");
			else
				sqlId = "";

			params.put("ROW_COUNT" + sqlId, Integer.toString(numbOfCols));

			for (int ndx = 0; rs.next(); ndx++)
			{
				for (int i = 1; i <= numbOfCols; i++)
				{
					colName = preFix + rsmd.getColumnName(i);
					int sqlTyp = rsmd.getColumnType(i);
					switch(sqlTyp)
					{
						case 12:	if (rs.getString(i) == null)
									val = "";
								else
									val =  rs.getString(i);
								params.put(colName + "_" + Integer.toString(ndx), 
										val);
								break;
						case 2:		params.put(colName + "_" + Integer.toString(ndx), 
										Integer.toString(rs.getInt(i))); 
								break;
						default: params.put("sysResultString", "svr.servletName.DBObject::Get() - " +
									" unsupported column type " + Integer.toString(sqlTyp) + 
									" for column name: " + colName);
							 params.put("sysResultStatus", "-1");
					}

					System.out.println("Col Name: " + colName + " is of SQL type: " + Integer.toString(sqlTyp));
				}
			}
                } catch (SQLException ex) {
                        params.put("sysResultStatus", "-1");
                        params.put("sysResultString", "TBDB.DBObject::Get() - SQLException: " +
                                        ex.getMessage() + " SQL Statement was: " + sqlStatement);
                }

		return 0;
	}
}

