
//-----------------------------------------------------------
//
//	Softbolts Ltd. (c) all rights reserved
//
//	Warning: This  program is  protected by copyright law 
//		 and   international  treaties.  Unauthorised
//		 reproduction or distribution of this program,
//		 or  any portion  or it, may result in severe
//		 civil  and  criminal  penalties, and will be
//		 prosecuted to  the  maximum extent possible.
//
//              Author: A. E. Dawson, 22 Apr 2003
//              mailto: anthony@softbolts.com
//
//		Last Revised: 22 Apr 2003
//
//-----------------------------------------------------------


package com.softbolts.connectors.TBDB;

import com.softbolts.lib.*;
import java.io.*;
import java.net.*;
import java.util.*;
import java.lang.Throwable;
import java.sql.*;

public class Servlet extends BaseServlet
{
 private static final int MIN_POLLTIME = 1;
 public PollerState pollerState = new PollerState();
 static final String keyLogin = "Login";
 static final String keyPassword = "Password";
 static final String keyDatabase = "Database Id";
 static final String keyHostName = "Host Name";
 static final String keyHostPort = "Host Port";
 static final String keyJdbcDriver = "JDBC Driver";
 static final String keySourceOrTarget = "Source/Target (S/T)";
 static final String keyTable = "Trigger Table";
 //static final String keyGetSql = "Get Sql Statement (source only)";
 static final String keySql = "Sql Statement (target only)";
 static final String keyPollingInterval = "Polling Interval (source only)";
 static final String keyExecuteModel = "Execute model (source only)";
 Vector tableNames = new Vector(10);
 String connectionStr = "";
 String executeModel = "";
 String login = "";
 String passwd = "";
 String hostName = "";
 boolean autoCommit = false;
 TDBInfo di = new TDBInfo();
 DBObject dbObj = null;
 int pi = 0;
 Connection dbCon = null;
 Statement stmt = null;
 ResultSet rset = null;

	public Servlet(String args[])
	{
		System.out.println("TBDB Connector Started...");

		servletName = "TBDB";
		servletType = SvrTyp.connector;

		servletState = RunStatusTyp.halted;
		if (servletState == RunStatusTyp.running)
			System.out.println("TBDB Connector status is running...");
		else if (servletState == RunStatusTyp.halted)
			System.out.println("TBDB Connector status is halted...");
		else
			System.out.println("TBDB Connector status is undefined...");
	
		procMode = ProcModeTyp.single_threaded;

		AddConfigParam(keyLogin, 20, "^[A-Za-z0-9_]*$", true);
		AddConfigParam(keyPassword, 20, "^[A-Za-z0-9_]*$", false);
		AddConfigParam(keyDatabase, 20, "^[A-Za-z0-9_]*$", true);
		AddConfigParam(keyHostName, 20, "^[A-Za-z0-9_]*$", true);
		AddConfigParam(keyHostPort, 10, "^[A-Za-z0-9_]*$", true);
		AddConfigParam(keyJdbcDriver, 50, "^[A-Za-z0-9_]*$", true);
		AddConfigParam(keySourceOrTarget, 2, "^[A-Za-z0-9_]*$", true);
		AddConfigParam(keyTable, 128, "^[A-Za-z0-9_]*$", true);
		//AddConfigParam(keyGetSql, 500, "^[A-Za-z0-9_]*$", true);
		AddConfigParam(keySql, 500, "^[A-Za-z0-9_]*$", true);
		AddConfigParam(keyPollingInterval, 5, "^[A-Za-z0-9_]*$", true);
		AddConfigParam(keyExecuteModel, 20, "^[A-Za-z0-9_]*$", true);
	}

        //-----------------------------------------------------------
        public void service(NameValueTyp params) throws ThrowableString
        {
         String sysService = null;

                if (params.containsKey("sysService"))
                        sysService = (String) params.get("sysService");

                System.out.println("TBDB.Servlet::service() - " + sysService);
                if (params.containsKey("sysService") && ((String) params.get("sysService")).equals("Halt Servlet"))
                        stopPoller();
        }

	//-----------------------------------------------------------
	public void init(NameValueTyp params) throws ThrowableString
	{
		System.out.println("TBDB.Servlet::init()");

		
		// check that the minimum configuration has been made
		if (((configParams.get(keyLogin).value.equals("")) == true) ||
			((configParams.get(keyPassword).value.equals("")) == true) ||
			((configParams.get(keyDatabase).value.equals("")) == true) ||
			((configParams.get(keyHostName).value.equals("")) == true) ||
			((configParams.get(keyHostPort).value.equals("")) == true) ||
			((configParams.get(keyJdbcDriver).value.equals("")) == true) ||
			((configParams.get(keySourceOrTarget).value.equals("")) == true))
		{
			System.out.println("TBDB.Servlet::init() - minimum configuration is not complete");
			servletState = RunStatusTyp.halted;
			throw new ThrowableString("TBDB.Servlet::init() - minimum configuration is not complete");
		}


		if ((configParams.get(keySourceOrTarget).value.toUpperCase().equals("S")) && 
			//((configParams.get(keyGetSql).value.equals("")) == false) && 
			((configParams.get(keyTable).value.equals("")) == false) &&
			((configParams.get(keyExecuteModel).value.equals("")) == false) &&
			((configParams.get(keyPollingInterval).value.equals("")) == false))
		{
			System.out.println("TBDB.Servlet::init() - acting as source");

			if (pollerState.test() == true) 
			{
				System.out.println("TBDB.Servlet::init() - Stopping Poller");
				stopPoller();
			}
			else
				System.out.println("TBDB.Servlet::init() - NOT Stopping Poller");

			try {
				initDBConnection();
			} catch (ThrowableString ex) {
				System.out.println(ex.getMessage());
				servletState = RunStatusTyp.halted;
				throw new ThrowableString(ex.getMessage());
			}
			addSchema();
			pi = getPollingInterval();
			String table = getTable();
			String shadowTableName = "tb_" + table + "_shd";
			String triggerName = "tb_" + table + "_trig";
			try {
				//executeModel = getExecuteModel();
				executeModel = Utils.verifyModel(this, configParams.get(keyExecuteModel).value);
				setupShadowTable(table, shadowTableName, triggerName);
			} catch (ThrowableString ex) {
				System.out.println(ex.getMessage());
				servletState = RunStatusTyp.halted;
				throw new ThrowableString(ex.getMessage());
			}
			startPoller(pi, executeModel, shadowTableName, triggerName, table);

		//} else if ((configParams.get(keySourceOrTarget).value.toUpperCase().equals("T")) &&
			//!(((configParams.get(keyTable).value.equals("")) == false) ||
			//((configParams.get(keySql).value.equals("")) == false)))
		} else if (configParams.get(keySourceOrTarget).value.toUpperCase().equals("T"))
		{
			System.out.println("TBDB.Servlet::init() - acting as target");

					// TO DO replace with the connection poller
			try {
				initDBConnection();
			} catch (ThrowableString ex) {
				System.out.println(ex.getMessage());
				servletState = RunStatusTyp.halted;
				throw new ThrowableString(ex.getMessage());
			}
			//String table = getTable();
			addSchema();
		}
		else
		{
			System.out.println("TBDB.Servlet::init() - minimum configuration is not complete");
			servletState = RunStatusTyp.halted;
			throw new ThrowableString("TBDB.Servlet::init() - minimum configuration is not complete");
		}
System.out.println("Done init()");
	}

	//-----------------------------------------------------------
	public void setupShadowTable(String table, String shadowTableName, String triggerName) throws ThrowableString
	{
	 String statement = null;

System.out.println(shadowTableName);
System.out.println(triggerName);
		try {
			statement = "drop table " + shadowTableName;
                        stmt.executeQuery(statement);
                } catch (SQLException ex) {
                        System.err.println("Deleting shadow table - TBDB.Servlet::setupShadowTable() - SQLException: " + 
					ex.getMessage() + " SQL Statement was: " + statement);
                }

		statement = "create table " + shadowTableName + "(" + getColumnSpec(table) + ")";
		try {
System.out.println("SHADOW TABLE SQL: " + statement);
                        stmt.executeQuery(statement);
                        stmt.executeQuery("commit");
                } catch (SQLException ex) {
System.out.println("FKING HELL1");
                        throw new ThrowableString("creating shadow table TBDB.Servlet::setupShadowTable() - SQLException: " + 
					ex.getMessage() + " SQL Statement was: " + statement);
                }

		TColumn col = null;
		TTable tab = di.td.get(table);
		Enumeration ce = tab.keys();
		String colList = null;
		while (ce.hasMoreElements ())
		{
			colList += ":new.";
			colList += (String) ce.nextElement();
			colList += ",";
		}

		statement = "create or replace trigger " + triggerName + " before insert or update on " +
			table + " for each row begin insert into " + shadowTableName + " values(" + 
			colList.substring(0, colList.length() - 1) + "); end;";
System.out.println("TRIGGER SQL: " + statement);
		try {
                        stmt.executeQuery(statement);
                        stmt.executeQuery("commit");
                } catch (SQLException ex) {
                        throw new ThrowableString("creating trigger on table TBDB.Servlet::setupShadowTable() - SQLException: " +
							ex.getMessage() + " SQL Statement was: " + statement);
                }
System.out.println("Created shadow table and trigger OK");
System.out.println(shadowTableName);
System.out.println(triggerName);
		//throw new ThrowableString("");
	}

	//-----------------------------------------------------------
	public void printSqlWarnings(String table) throws ThrowableString
	{
/*
	 SQLWarning w =  null;

		try {
			w = dbCon.getWarnings();
			do {
				System.out.println(shadowTableName);
			} while (w = w.getNextWarning());

                } catch (SQLException ex) {
                        throw new ThrowableString("creating trigger on table TBDB.Servlet::setupShadowTable() - SQLException: " +
							ex.getMessage() + " SQL Statement was: " + statement);
		
*/
	}

	//-----------------------------------------------------------
	public String getColumnSpec(String table) throws ThrowableString
	{
		TColumn colDet = null;
		TTable tab = di.td.get(table);
		Enumeration ce = tab.keys();
		String colList = "";
		String colName = null;

						// may need to add SCALE, REFERENCE, NULLS?, 
						// DEFAULT VALUE, SCOPESCHEMA, SCOPE TABLE
		while (ce.hasMoreElements())
		{
			colName = (String) ce.nextElement();
			colDet = tab.get(colName);
			if (colDet.type_name.equals("VARCHAR2"))
			{
				colList += "\"" + colName + "\" VARCHAR2(" +
                                                        Integer.toString(colDet.column_size) + ")";

			} else if (colDet.type_name.equals("NUMBER"))
			{
				colList += colName + " NUMBER(" + Integer.toString(colDet.column_size) +
                                                        "," + colDet.decimal_digits + ")";
			} else if (colDet.type_name.equals("NCHAR"))
			{
				colList += "\"" + colName + "\" NCHAR(" +
                                                        Integer.toString(colDet.column_size) + ")";
			} else if (colDet.type_name.equals("CHAR"))
			{
				colList += "\"" + colName + "\" VARCHAR2(" +
                                                        Integer.toString(colDet.column_size) + ")";
			} else if (colDet.type_name.equals("FLOAT"))
			{
				colList += colName + " FLOAT";
			} else if (colDet.type_name.equals("UROWID"))
			{
				colList += colName + " UROWID";
			} else if (colDet.type_name.equals("BFILE"))
			{
				colList += colName + " BFILE";
			} else if (colDet.type_name.equals("NCLOB"))
			{
				colList += colName + " NCLOB";
			} else if (colDet.type_name.equals("RAW"))
			{
				colList += "\"" + colName + "\" RAW(" +
                                                        Integer.toString(colDet.column_size) + ")";
			} else if (colDet.type_name.equals("BLOB"))
			{
				colList += colName + " BLOB";
			} else if (colDet.type_name.equals("LONG"))
			{
				colList += colName + " LONG";
			} else if (colDet.type_name.equals("DATE"))
			{
				colList += colName + " DATE";
			} else if (colDet.type_name.equals("ROWID"))
			{
				colList += colName + " ROWID";
			} else if (colDet.type_name.equals("NVARCHAR2"))
			{
				colList += "\"" + colName + "\" NVARCHAR2(" +
                                                        Integer.toString(colDet.column_size) + ")";
			}
			
			colList += ",";
		}

		return colList.substring(0, colList.length() - 1);		// remove the final comma
	}

	//-----------------------------------------------------------
	public void addSchema() throws ThrowableString
	{
	 TTableDescriptions td = di.td;
	 String tableName = null;
	 String columnName = null;
	 TTable tab = null;
	 TColumn col = null;

		dbObj = new DBObject(this);
		ClearObjects();
//System.out.println("addSchema()");

		Enumeration te = td.keys();
                while (te.hasMoreElements())
                {
                        tableName = (String) te.nextElement();
			AddObject(tableName);
			AddMethod(tableName, "Get", dbObj);
			AddMethod(tableName, "Set", dbObj);
//System.out.println("Adding Table-> " + tableName);

			tab = td.get(tableName);
	 		Enumeration ce = tab.keys();
			while (ce.hasMoreElements())
			{
				columnName = (String) ce.nextElement();
//col  = tab.get(columnName);
//System.out.println("Col Type name: " + col.type_name);
//System.out.println("Col name: " + columnName + " type id: " + col.data_type);
				AddField(tableName, columnName);
//System.out.println("Adding -> " + tableName + "." + columnName);

//				col = (TColumn) tab.get(columnName);
//				int size = col.column_size;
//System.out.println("Column size-> " + size);
			}
                }
//System.out.println("addSchema() : DONE");
	}


	//-----------------------------------------------------------
	public String getHostName() throws ThrowableString
	{
		return configParams.get(keyHostName).value.toLowerCase();
	}

	//-----------------------------------------------------------
	public String getSetSql() throws ThrowableString
	{
		return configParams.get(keySql).value.toUpperCase();
	}

	//-----------------------------------------------------------
	public String getTable() throws ThrowableString
	{
		String table = configParams.get(keyTable).value.toUpperCase();
		if (di.td.containsKey(table.toUpperCase()) == false)
			throw new ThrowableString("Table name: " + table + " does not exist!");

		return table;
	}

	//-----------------------------------------------------------
	public void initDBConnection() throws ThrowableString
	{
	 //Connection dbCon = null;

		System.out.println("TBDB.Servlet::initDBConnection()");
			

		connectionStr = configParams.get(keyJdbcDriver).value +
				":@" +
				configParams.get(keyHostName).value +
				":" +
				configParams.get(keyHostPort).value +
				":" +
				configParams.get(keyDatabase).value;
				
		//System.out.println("Using connection string: " + connectionStr);
		login = configParams.get(keyLogin).value;
		passwd = configParams.get(keyPassword).value;

		try {
			if (dbCon != null)
			{
				dbCon.close();
				dbCon = null;
			}
			DriverManager.registerDriver(new oracle.jdbc.driver.OracleDriver());
System.out.println("Registered Oracle Driver");
			//dbCon = DriverManager.getConnection ("jdbc:oracle:thin:@ganymede:1521:CCDB1", "csr", "andiamoa");
			dbCon = DriverManager.getConnection (connectionStr, login, passwd);
System.out.println("Got a connection to the DB");
			autoCommit = dbCon.getAutoCommit();
		System.out.println("Autocommit status is: " + autoCommit);
			stmt = dbCon.createStatement();
			getTables(dbCon);

		} catch (SQLException ex) {
			throw new ThrowableString("Servlet::initDBConnection() - " + ex.getMessage() + " to: " + 
					connectionStr + " with credentials: " + login + "/" + passwd);
		}
	}


	//-----------------------------------------------------------
	public void getTables(Connection conn) throws SQLException
	{
	 boolean found = false;

		rset = stmt.executeQuery("select table_name from tabs");
		tableNames.clear();
		while (rset.next())
			tableNames.add(rset.getString(1));

		rset.close();

		for (int i = 0; i < tableNames.size(); i++)
			getColumnDescriptions(conn, rset, (String) tableNames.get(i));
	}

	//-----------------------------------------------------------
	public void getColumnDescriptions(Connection conn, ResultSet rset, String tableName) throws SQLException
	{
	 TTableDescriptions td = di.td;
	 TTable aTable = new TTable();
	 DatabaseMetaData dbmd = null;
	
		td.put(tableName, aTable);
		dbmd = conn.getMetaData();
		di.isReadOnly = dbmd.isReadOnly();
		rset = dbmd.getColumns(null, null, tableName, null);
		while (rset.next())
		{
			TColumn colDet = new TColumn();
			colDet.table_cat = rset.getString(1);
			colDet.table_schem = rset.getString(2);
			colDet.table_name = rset.getString(3);
			colDet.column_name = rset.getString(4);
			colDet.data_type = rset.getInt(5);
			colDet.type_name = rset.getString(6);
			colDet.column_size = rset.getInt(7);
			colDet.decimal_digits = rset.getInt(9);
			colDet.num_prec_radix = rset.getInt(10);
			colDet.nullable = rset.getInt(11);
			colDet.remarks = rset.getString(12);
			colDet.column_def = rset.getString(13);
			colDet.sql_data_type = rset.getInt(14);
			colDet.sql_datetime_sub = rset.getInt(15);
			colDet.char_octed_length = rset.getInt(16);
			colDet.ordinary_position = rset.getInt(17);
			colDet.is_nullable = rset.getString(18);
			//-----------------------------------------------------------
			// This lot doesn't appear to exist in the Oracle thin driver
			//colDet.scope_catalogue = rset.getString(19);
			//colDet.scope_schema = rset.getString(20);
			//colDet.scope_table = rset.getString(21);
			//colDet.source_data_type = rset.getShort(22);

			aTable.put(colDet.column_name, colDet);
//System.out.println("getColumnDescriptions() - " + colDet.column_name);
		}
		rset.close();
		System.out.println("DONE");
	}

	//-----------------------------------------------------------
	public String getExecuteModel() throws ThrowableString
	{
	 boolean validModel = false;
	 String model = configParams.get(keyExecuteModel).value;

		if (model.equals("") == false)
		{
			NameValueTyp tmpMap = new NameValueTyp();
			tmpMap.put("sysRequest", "List Models");
			tmpMap.put("sysClientId", assignedName);
			tmpMap.put("sysDebug", "True");
			tmpMap.put("sysReply", "True");

			SocketObj sok = null;
			try {
				sok = new SocketObj(enginePort, engineHostName);
			} catch (IOException ioe) {
				System.err.println("Servlet::getExecuteModel() - No Engine Found " + ioe.getMessage());
			}
			if (sok.connected)
			{
				sok.write(tmpMap);
				sok.read(tmpMap);
				sok.close();
			}
			else // (!sok.connected)
			{
				tmpMap.put("sysResultStatus", "-1");
				tmpMap.put("sysResultString", "Socket Connection failed");
			}

			if (tmpMap.containsKey("sysResultStatus") && tmpMap.get("sysResultStatus").equals("-1"))
			{
				System.err.println("Servlet::getExecuteModel() - Cannot verify configured transaction model " +
							 tmpMap.get("sysResultString"));
				validModel = true;
			}
			
			String modelName = null;
			for (int i = 0; tmpMap.containsKey("sysModelName" + Integer.toString(i)); i++)
			{
				modelName = (String) tmpMap.get("sysModelName" + Integer.toString(i));
				if (modelName.equals(model))
				{
					validModel = true;
					System.err.println("Servlet::getExecuteModel() - Configured model verified OK");
				}
			} 
			if (validModel == false)
				throw new ThrowableString("Servlet::getExecuteModel() - Execute model does not exist");
		}
		return(model);
	}

	//-----------------------------------------------------------
	public void startPoller(int pi, String model, String shadowTableName, String triggerName, String table) throws ThrowableString
	{
											// fork off the poller thread
		System.err.println("Servlet::startPoller() - Attempting to start poller");
		System.err.println("Servlet::startPoller() - pi             : " + pi);
		System.err.println("Servlet::startPoller() - model          : " + model);
		System.err.println("Servlet::startPoller() - table          : " + shadowTableName);
		System.err.println("Servlet::startPoller() - triggerName    : " + triggerName);


		if (pollerState.test() == false)
		{
			System.err.println("Servlet::init() - Starting poller");
			//System.out.println("Servlet::init() - Starting poller");
			DBPoller dbp = new DBPoller(pollerState, pi, model, assignedName, connectionStr, shadowTableName, triggerName,
							enginePort, engineHostName, di, table, login, passwd);
			dbp.start();
		}
		else	// should never happen as long as stopPoller has worked properly!
		{
			System.err.println("Servlet::init() - Poller already running");
			throw new ThrowableString("Servlet::init() - Poller already running");
		}
	}

        //-----------------------------------------------------------
        public void stopPoller()
        {
	 Thread cth = Thread.currentThread();

                if (pollerState.test() == true)
                {
								// tell the pooler to stop
								// once stopped the poller will 
								// indicate this by setting the 
								// pollerState flag
                        pollerState.reset();
			while (pollerState.test() == false)
			{
				try {
					cth.sleep(1000);	// thousands of one second
				} catch (InterruptedException ie) {
					System.err.println("TBDB.FilePoller::run() - InterruptedException on call to sleep()");
				}
			}
                        pollerState.reset();
                }
        }

        //-----------------------------------------------------------
        public int getPollingInterval() throws ThrowableString
        {
         int pi = MIN_POLLTIME;

                if (configParams.get(keyPollingInterval).value.equals("") == false)
                {
                        String intString = configParams.get(keyPollingInterval).value;
			pi = Integer.parseInt(intString);
                }

                if (pi < MIN_POLLTIME) pi = MIN_POLLTIME;

                return (pi);
        }

	//-----------------------------------------------------------
	public void finish(NameValueTyp params) throws ThrowableString
	{
		System.err.println("TBDB.Servlet::finish()");
		stopPoller();
		System.out.println("TBDB Connector Stopped...");
	}

	//-----------------------------------------------------------
	public void beginTransaction(NameValueTyp params) throws ThrowableString
	{
	}

	//-----------------------------------------------------------
	public void commitTransaction(NameValueTyp params) throws ThrowableString
	{
	}

	//-----------------------------------------------------------
	public void rollBackTransaction(NameValueTyp params) throws ThrowableString
	{
	}
}


