
//-----------------------------------------------------------
//
//	Softbolts Ltd. (c) all rights reserved
//
//	Warning: This  program is  protected by copyright law 
//		 and   international  treaties.  Unauthorised
//		 reproduction or distribution of this program,
//		 or  any portion  or it, may result in severe
//		 civil  and  criminal  penalties, and will be
//		 prosecuted to  the  maximum extent possible.
//
//              Author: A. E. Dawson, March 2003
//              mailto: anthony@softbolts.com
//
//		Last Revised: 4th March 2003
//
//-----------------------------------------------------------

package com.softbolts.connectors.TBFile;

import com.softbolts.lib.*;

import java.io.*;
import java.util.*;
import java.lang.*;
import java.lang.reflect.*;


public class FilePoller extends Thread 
{
 PollerState pollerState = null;
 String transName = null;
 String directory = null;
 String targetMachine = null;
 String targetPort = null;
 boolean binary = false;
 boolean encrypt = false;
 int pollingInterval = 0;
 int enginePort = 0;
 String engineHostName = null;
 String instanceName = null;

        //-----------------------------------------------------------
        public FilePoller(int enginePort, String engineHostName, PollerState pollerState, String directory, String transName, 
			String mode, String encrypt, String targetMachine, String TargetPort, 
			int pollingInterval, String instanceName) throws ThrowableString
        {
		this.pollerState = pollerState;
		this.transName = transName;
		this.directory = directory;
		this.binary = mode.equals("B");
		this.encrypt = encrypt.equals("Y");
		this.pollingInterval = pollingInterval;
		this.targetMachine = targetMachine;
		this.targetPort = targetPort;
		this.enginePort = enginePort;
		this.engineHostName = engineHostName;
		this.instanceName = instanceName;
		pollerState.set();
        }

        //-----------------------------------------------------------
        public void run()
        {
	 File fileDir = null;

		System.err.println("FilePoller->run");
		if (encrypt)
		{
			System.err.println("FilePoller::run() - encryption not yet available");
			return;
		}

		if (binary)
		{
			System.err.println("FilePoller::run() - Binary mode not yet supported");
			return;
		}

		System.err.println("Starting Poller with transName = " + transName + " directory = " + directory + 
					" polling interval = " + pollingInterval);

		while (pollerState.test())
		{
			System.err.println(instanceName + " Polling...");

			File aDir = new File(directory);
			String fileList[] = aDir.list();

			for (int i = 0, n = fileList.length; i < n; i++)
			{
				System.err.println("Processing file: " + fileList[i]);

				NameValueTyp tmpMap = new NameValueTyp();
				tmpMap.put("sysTransName", transName);
				tmpMap.put("File Name", fileList[i]);
				tmpMap.put("sysClientId", "TBFile");
				tmpMap.put("sysTimeout", "20");
				tmpMap.put("sysDebug", "True");
				tmpMap.put("sysReply", "True");

				SocketObj sok = null;
				try {
					sok = new SocketObj(enginePort, engineHostName);
					if (sok.connected)
					{
						sok.write(tmpMap);
						sok.read(tmpMap);
						sok.close();
					}
					else // (!sok.connected)
					{
						tmpMap.put("sysResultStatus", "-1");
						tmpMap.put("sysResultString", "FilePoller::run() - Socket Connection failed");
						System.err.println((String) tmpMap.get("sysResultString"));
					}
				} catch (IOException ioe) {
					System.err.println("FilePoller::run() - No Engine Found " + ioe.getMessage());
				} catch (ThrowableString s) {
					System.err.println("FilePoller::run() -  " + s.getMessage());
				}

				PersistStream.printMap(tmpMap);
			}
			
			try {
				sleep(pollingInterval * 1000);		// thousands of one second
			} catch (InterruptedException ie) {
				System.err.println("FilePoller::run() - InterruptedException on call to sleep()");
			}
		}
		System.err.println("FilePoller::run() - Poller stopping");
	}
}

