
//-----------------------------------------------------------
//
//	Softbolts Ltd. (c) all rights reserved
//
//	Warning: This  program is  protected by copyright law 
//		 and   international  treaties.  Unauthorised
//		 reproduction or distribution of this program,
//		 or  any portion  or it, may result in severe
//		 civil  and  criminal  penalties, and will be
//		 prosecuted to  the  maximum extent possible.
//
//              Author: A. E. Dawson, 15 Oct 1999
//              mailto: anthony@softbolts.com
//
//		Last Revised: 15 Oct 2002
//
//-----------------------------------------------------------


package com.softbolts.connectors.TBFile;

import com.softbolts.lib.*;
import java.io.*;
import java.net.*;
import java.util.*;
import java.lang.Throwable;

public class Servlet extends BaseServlet
{
 private static final int MIN_POLLTIME = 1;
 public PollerState pollerState = new PollerState();
 static final String keySourceOrTarget = "Source or Target (S or T)";
 static final String keyDirectoryName = "Directory name";
 static final String keyExecuteModel = "Execute model";
 static final String keyBinaryOrText = "Binary or Text (B or T)";
 static final String keyPollingInterval = "PollingInterval";
 static final String keyMachineName = "Machine Name (Binary Only)";
 static final String keyPortNumber = "Port Number (Binary Only)";
 static final String keyUseEncryption = "Use Encryption(Y or N) (Binary Only)";


	public Servlet(String args[])
	{
		System.out.println("TBFile Connector Starting...");

		FileObj fo = new FileObj(this);

		servletName = "TBFile";
		servletType = SvrTyp.connector;
		servletState = RunStatusTyp.running;

		AddConfigParam(keyDirectoryName, 128, "", true);
		AddConfigParam(keySourceOrTarget, 1, "", true);
		AddConfigParam(keyExecuteModel, 50, "", true);
		AddConfigParam(keyPollingInterval, 2, "", true);
		AddConfigParam(keyBinaryOrText, 1, "", true);
		AddConfigParam(keyMachineName, 20, "", true);
		AddConfigParam(keyPortNumber, 10, "", true);
		AddConfigParam(keyUseEncryption, 10, "", true);


		AddObject("File");
		AddMethod("File", "Get", fo);
		AddMethod("File", "Set", fo);
		AddMethod("File", "Stop poller", fo);
	}

	//-----------------------------------------------------------
	public void stopPoller()
	{
		if (pollerState.test() == true)
		{
			pollerState.reset();
		}
	}

	//-----------------------------------------------------------
	public void service(NameValueTyp params) throws ThrowableString
	{
	 String sysService = null;

		if (params.containsKey("sysService"))
			sysService = (String) params.get("sysService");

		System.out.println("TBFile.service() - " + sysService);
		if (params.containsKey("sysService") && ((String) params.get("sysService")).equals("Halt Servlet"))
			stopPoller();
	}

	//-----------------------------------------------------------
	public void init(NameValueTyp params) throws ThrowableString
	{
	 String directory = null;
	 String executeModel = null;
	 String transferMode = null;
	 String encryptionSetting = null;
	 String sourceMachine = null;
	 String targetMachine = null;
	 String sourcePort = null;
	 String targetPort = null;
	 int pi = 0;

		//System.out.println("TBFile.init()");

		//boolean piSet = false;
		//boolean modeSet = false;


		//String executeModel = null;
		//String directory = null;
		//String encryptionSetting = null;

		if (pollerState.test() == true) stopPoller();

		String mode = ((ServletParamInfo) configParams.get(keySourceOrTarget)).value;

		if (mode.toUpperCase().equals("S"))			// acting as a source connector
		{							// delete the file after the copy
									// For a source to work properly we need
									// 1) a valid readable directory
									// 2) binary or Text transfer setting
									// 3) a valid model to execute
									// 4) a valid setting for incryption
			directory = getDirectory(params);
			executeModel = getExecuteModel(params);
			encryptionSetting = getEncryptionSetting(params);
			pi = getPollingInterval(params);
			transferMode = getTransferMode(params);

			if (transferMode != null && transferMode.equals("B"))
			{
				targetMachine = getTargetMachine(params);
				targetPort = getTargetPort(params);
			}

			//boolean binaryOK = (transferMode.equals("B") && ((targetMachine != null) && (targetPort != null)));
		
			if (servletState != RunStatusTyp.running)
				System.out.println("Servlet::init() - Bugger");

			if (servletState == RunStatusTyp.running &&
			    (directory != null) && 
			    (executeModel != null) && 
			    (transferMode != null) && 
			    (encryptionSetting != null) &&
			    (transferMode.equals("T") || ((targetMachine != null) && (targetPort != null))))
				startPoller(directory, executeModel, transferMode, 
					    encryptionSetting, targetMachine, targetPort, pi);
		}
		else if (mode.toUpperCase().equals("T"))		// acting as the target connector
		{
		}

	}

	//-----------------------------------------------------------
	public void startPoller(String dir, String model, String mode, String encrypt, String targetMachine,
				String targetPort, int pi) throws ThrowableString
	{
		// fork off the poller process
		System.err.println("Servlet::init() - Attempting to start poller");
		System.err.println("Servlet::init() - directory: " + dir);
		System.err.println("Servlet::init() - model    : " + model);
		System.err.println("Servlet::init() - mode     : " + mode);
		System.err.println("Servlet::init() - encrypt  : " + encrypt);
		System.err.println("Servlet::init() - pi       : " + pi);

		if (pollerState.test() == true) stopPoller();

		if (pollerState.test() == false)
		{
			System.err.println("Servlet::init() - Starting poller");
			FilePoller fp = new FilePoller(enginePort, engineHostName, pollerState, 
						       dir, model, mode, encrypt, targetMachine, 
						       targetPort, pi, assignedName);
			fp.start();
		}
		else
		{
			System.err.println("Servlet::init() - Poller already running");
			throw new ThrowableString("Servlet::init() - Poller already running");
		}
	}

	//-----------------------------------------------------------
	public String getTargetMachine(NameValueTyp params) throws ThrowableString
	{
		System.err.println("Servlet::getTargetMachine() - Called");
		return(null);
	}

	//-----------------------------------------------------------
	public String getTargetPort(NameValueTyp params) throws ThrowableString
	{
		System.err.println("Servlet::getTargetPort() - Called");
		return(null);
	}

	//-----------------------------------------------------------
	public String getDirectory(NameValueTyp params) throws ThrowableString
	{
		String directory = ((ServletParamInfo) configParams.get(keyDirectoryName)).value;
		if (directory.equals("") == false)
		{
			File aDir = new File(directory);
			if (aDir.isDirectory() == false)
				 throw new ThrowableString("Servlet::checkDirParam() - " + directory + " is not a directory");

			if (((ServletParamInfo) configParams.get(keySourceOrTarget)).value.toUpperCase().equals("S"))
			{
				if (aDir.canRead() == false)
					throw new ThrowableString("Servlet::checkDirParam() - no read permission on " + 
									directory);
			}
			else if (((ServletParamInfo) configParams.get(keySourceOrTarget)).value.toUpperCase().equals("T"))
			{
				if (aDir.canWrite() == false)
					throw new ThrowableString("Servlet::checkDirParam() - no permission permission on " + 
									directory);
			}
			return(directory);
		}

		return(null);
	}

	//-----------------------------------------------------------
	public String getEncryptionSetting(NameValueTyp params) throws ThrowableString
	{
		String encrypt = ((ServletParamInfo) configParams.get(keyUseEncryption)).value;
		if (encrypt.toUpperCase().equals("Y"))
			return("Y");
		else if (encrypt.toUpperCase().equals("N"))
			return("N");
/*
		throw new ThrowableString("Servlet::getTransferMode() - invalid encryption mode >" +
							encrypt + "< specified");
*/
		return(null);
	}

	//-----------------------------------------------------------
	public String getExecuteModel(NameValueTyp params) throws ThrowableString
	{
	 boolean validModel = false;
	 String model = ((ServletParamInfo) configParams.get(keyExecuteModel)).value;

		if (model.equals("") == false)
		{
			NameValueTyp tmpMap = new NameValueTyp();
			tmpMap.put("sysRequest", "List Models");
			tmpMap.put("sysClientId", assignedName);
			tmpMap.put("sysDebug", "True");
			tmpMap.put("sysReply", "True");

			SocketObj sok = null;
			try {
				sok = new SocketObj(enginePort, engineHostName);
			} catch (IOException ioe) {
				System.err.println("Servlet::getExecuteModel() - No Engine Found " + ioe.getMessage());
			}
			if (sok.connected)
			{
				sok.write(tmpMap);
				sok.read(tmpMap);
				sok.close();
			}
			else // (!sok.connected)
			{
				tmpMap.put("sysResultStatus", "-1");
				tmpMap.put("sysResultString", "Socket Connection failed");
			}

			if (params.containsKey("sysResultStatus") && tmpMap.get("sysResultStatus").equals("-1"))
			{
				System.err.println("Servlet::checkModel() - Cannot verify configured transaction model " +
							 tmpMap.get("sysResultString"));
				validModel = true;
			}
			
			String modelName = null;
			for (int i = 0; tmpMap.containsKey("sysModelName" + Integer.toString(i)); i++)
			{
				modelName = (String) tmpMap.get("sysModelName" + Integer.toString(i));
				if (modelName.equals(model))
				{
					validModel = true;
					System.err.println("Servlet::checkModel() - Configured model verified OK");
				}
			} 
			if (validModel == false)
				throw new ThrowableString("Servlet::checkModel() - Execute model does not exist");
		}
		return(model);
	}

	//-----------------------------------------------------------
	public int getPollingInterval(NameValueTyp params) throws ThrowableString
	{
	 int pi = MIN_POLLTIME;

		if (((ServletParamInfo) configParams.get(keyPollingInterval)).value.equals("") == false)
		{ 
			String intString = ((ServletParamInfo) configParams.get(keyPollingInterval)).value;
			// try {
				pi = Integer.parseInt(intString);
/*
			 } catch (...) {
				System.err.println("Servlet::pollingInterval() - parseInt() failed for string >" +
							intString + "<");
				pi = MIN_POLLTIME;
			 }
*/
		}

		if (pi < MIN_POLLTIME) pi = MIN_POLLTIME;
 
		return (pi);
	}

	//-----------------------------------------------------------
	public String getTransferMode(NameValueTyp params) throws ThrowableString
	{
		String transferMode = ((ServletParamInfo) configParams.get(keyBinaryOrText)).value;
		if (transferMode.toUpperCase().equals("T"))
			return("T");
		else if (transferMode.toUpperCase().equals("B"))
			return("B");
/*
		throw new ThrowableString("Servlet::getTransferMode() - invalid transfer mode >" +
						transferMode + "< specified");
*/
		return(null);
	}

	//-----------------------------------------------------------
	public void finish(NameValueTyp params) throws ThrowableString
	{
		System.out.println("TBFile.finish()");
		stopPoller();
		System.out.println("TBFile Connector Stopped...");
	}

	//-----------------------------------------------------------
	public void beginTransaction(NameValueTyp params) throws ThrowableString
	{
		System.err.println("Servlet::beginTransaction() - called");
		String mode = ((ServletParamInfo) configParams.get(keySourceOrTarget)).value;

		if (mode.toUpperCase().equals("S"))			// acting as a source connector
		{							// check file exists and is readable
			if (params.containsKey("File Name") == false)
				 throw new ThrowableString("Servlet::beginTransaction() - missing File Name parameter ");

			if (params.containsKey("sysTransaction") == false)
				 throw new ThrowableString("Servlet::beginTransaction() - missing sysTransaction parameter ");

			String fn = ((ServletParamInfo) configParams.get(keyDirectoryName)).value + "/" + 
					(String) params.get("File Name");
			File aFile = new File(fn);
			if (aFile.canRead() == false)
				throw new ThrowableString("Servlet::init() - Can't read source file " + fn);
		}
		else if (mode.toUpperCase().equals("T"))		// acting as a target connector
		{							// check directory exists and is writable
			String directory = ((ServletParamInfo) configParams.get(keyDirectoryName)).value;
			File aDir = new File(directory);
			if (aDir.isDirectory() == false)
				 throw new ThrowableString("Servlet::beginTransaction() - target directory " + 
								aDir + " doesn't exist or is not a directory");
			if (aDir.canWrite() == false)
				 throw new ThrowableString("Servlet::beginTransaction() - target directory " + 
						aDir + " does't have write permission");
		}
		else
			throw new ThrowableString("Servlet::beginTransaction() -  ERR: mode is neither source or target");
			//System.out.println("Servlet::beginTransaction() - ERR: mode is neither source or target");
	}

	//-----------------------------------------------------------
	public void commitTransaction(NameValueTyp params) throws ThrowableString
	{
		System.err.println("Servlet::commitTransaction() - called");
		String mode = ((ServletParamInfo) configParams.get(keySourceOrTarget)).value;
		if (mode.toUpperCase().equals("S"))			// acting as a source connector
		{							// delete the file after the copy
			if (params.containsKey("File Name") == false)
				 throw new ThrowableString("Servlet::commitTransaction() - missing File Name parameter ");

			String fn = ((ServletParamInfo) configParams.get(keyDirectoryName)).value + "/" + 
					(String) params.get("File Name");
			File aFile = new File(fn);
			if (aFile.delete() == false)
				throw new ThrowableString("Servlet::commitTransaction() - Can't delete file:  " + fn);
		}
		else							// acting as the target connector
		{
			// nothing to do here
		}
	}

	//-----------------------------------------------------------
	public void rollBackTransaction(NameValueTyp params) throws ThrowableString
	{
		System.err.println("Servlet::rollBackTransaction() - called");
		String mode = ((ServletParamInfo) configParams.get(keySourceOrTarget)).value;
		if (mode.toUpperCase().equals("S"))			// acting as a source connector
		{	
			// Nothing to do here
		}
		else							// acting as the target connector
		{
			if (params.containsKey("File Name"))
				 throw new ThrowableString("Servlet::commitTransaction() - missing File Name parameter ");

			if (params.containsKey("sysTransaction"))
				 throw new ThrowableString("Servlet::commitTransaction() - missing sysTransaction parameter ");

			String fn = ((ServletParamInfo) configParams.get(keyDirectoryName)).value + 
					"/" + (String) params.get("File Name");
			File aFile = new File(fn);
			if (aFile.canRead() == false)
				if (aFile.delete() == false)
					throw new ThrowableString("Servlet::rollBackTransaction() - Can't delete or non existent file: " 
									+ fn);
		}
	}
}


