
//-----------------------------------------------------------
//
//	Softbolts Ltd. (c) all rights reserved
//
//	Warning: This  program is  protected by copyright law 
//		 and   international  treaties.  Unauthorised
//		 reproduction or distribution of this program,
//		 or  any portion  or it, may result in severe
//		 civil  and  criminal  penalties, and will be
//		 prosecuted to  the  maximum extent possible.
//
//              Author: A. E. Dawson, 15 Oct 1999
//              mailto: anthony@softbolts.com
//
//		Last Revised: 15 Oct 2002
//
//-----------------------------------------------------------


package com.softbolts.connectors.TBHTTPSample;

import com.softbolts.lib.*;
import java.io.*;
import java.net.*;
import java.util.*;
import java.lang.Throwable;

public class Servlet extends BaseServlet
{
 boolean httpServletRunning = false;
 public PollerState keepRunning = new PollerState();
 static final String keyPassword = "Password";
 static final String keyLogin = "Login";
 static final String keyModel = "Model (triggered if not blank)";
 static final String keyHttpPort = "Http Port";
 int port = 0;

	public Servlet(String args[])
	{

		HTTPObj httpObj = new HTTPObj();

		servletName = "TBHTTPSample";
		System.out.println(servletName + " Connector Started...");
		servletType = SvrTyp.connector;
		servletState = RunStatusTyp.halted;

                if (servletState == RunStatusTyp.running)
                        System.out.println(servletName +" Connector status is running...");
                else if (servletState == RunStatusTyp.halted)
                        System.out.println(servletName + " Connector status is halted...");
                else
                        System.out.println(servletName + " Connector status is undefined...");

		protocol = ProtocolTyp.http;

		AddConfigParam(keyModel, 6, "^[A-Za-z0-9_]*$", true);
		AddConfigParam(keyHttpPort, 6, "^[A-Za-z0-9_]*$", true);

		AddObject("HTTPObj");
		AddField("HTTPObj", "Thing_Obj");

		AddMethod("HTTPObj", "Get", httpObj);
		AddMethod("HTTPObj", "Set", httpObj);
	}

        //-----------------------------------------------------------
        public void service(NameValueTyp params) throws ThrowableString
        {
		System.err.println(servletName + ".Servlet()::service()");
        }

	//-----------------------------------------------------------
	public void init(NameValueTyp params) throws ThrowableString
	{
	 String modelName = "";

		if (((configParams.get(keyModel).value.equals("")) == true) ||
		    (configParams.get(keyHttpPort).value.equals("")))
		{
			servletState = RunStatusTyp.halted;
			throw new ThrowableString(servletName + ".Servlet::init() - minimum configuration is not complete");
		}
		else 
		{
			modelName = Utils.verifyModel(this, configParams.get(keyModel).value);
		}

		try {
			port = Integer.parseInt(configParams.get(keyHttpPort).value);
		} catch (NumberFormatException nfe) {
			throw new ThrowableString(servletName + ".Servlet::init() - NumberFormatException: " + nfe.getMessage());
		}

		System.err.println(servletName + ".Servlet()::init()");
		if (keepRunning.test() == false)
		{
			HTTPServlet httpSvr = new HTTPServlet(port, modelName, keepRunning);
			httpSvr.start();
		}
	}

	//-----------------------------------------------------------
	public void finish(NameValueTyp params) throws ThrowableString
	{
		System.err.println(servletName + ".Servlet()::finish()");
		keepRunning.reset();
		try {
		    SocketObj unlock = new SocketObj(port, "ganymede");
		    unlock.close() ;
		} catch (IOException ex) {
		    System.err.println("IOException while unblocking server thread: " + ex.getMessage());
		}
		System.out.println(servletName + " Connector Closed...");
	}

	//-----------------------------------------------------------
	public void beginTransaction(NameValueTyp params) throws ThrowableString
	{
		System.err.println(servletName + ".Servlet()::beginTransaction()");
	}

	//-----------------------------------------------------------
	public void commitTransaction(NameValueTyp params) throws ThrowableString
	{
		System.err.println(servletName + ".Servlet()::commitTransaction()");
	}

	//-----------------------------------------------------------
	public void rollBackTransaction(NameValueTyp params) throws ThrowableString
	{
		System.err.println(servletName + ".Servlet()::rollBackTransaction()");
	}
}


