//-----------------------------------------------------------
//
//	Softbolts Ltd. (c) all rights reserved
//
//	Warning: This  program is  protected by copyright law 
//		 and   international  treaties.  Unauthorised
//		 reproduction or distribution of this program,
//		 or  any portion  or it, may result in severe
//		 civil  and  criminal  penalties, and will be
//		 prosecuted to  the  maximum extent possible.
//
//              Author: A. E. Dawson, Aug 1999
//              mailto: anthony@softbolts.com
//
//		Last Revised: 07 Nov 2002
//
//-----------------------------------------------------------

package com.softbolts.lib;

import java.io.*;
import java.net.*;
import java.util.*;
import java.lang.*;

public class RequestExecuter
{
 SocketObj sok;
 NameValueTyp r;
 DateTime dt;
 BaseServlet svr;
 QSemaphore tidFileSem;

	//-----------------------------------------------------------
	public RequestExecuter(QSemaphore tidFileSem, Socket s, BaseServlet svr, NameValueTyp r) throws ThrowableString
	{
		this.tidFileSem = tidFileSem;
		this.sok = new SocketObj(s);
		//this.svr = (BaseServlet) svr;
		this.svr = svr;
		this.r = r;
	}           

	//-----------------------------------------------------------
	private void processMsg() throws ThrowableString
	{
		System.err.println("In processMsg");

		if (r.size() < 2)
			throw new ThrowableString("insufficient parameters");

		if (!r.containsKey("sysClientId"))
			throw new ThrowableString("Missing sysClientId in input data");

									// perform the required service
		if (r.containsKey("sysMethod"))
			executeMethod();
		else if ((r.containsKey("sysRequest")) &&
			(r.get("sysRequest").equals("Get Servlet Spec")))
			servletSpec();
		else if ((r.containsKey("sysRequest")) &&
			(r.get("sysRequest").equals("Get Config Values")))
			getConfigValues();
		else if ((r.containsKey("sysRequest")) &&
			(r.get("sysRequest").equals("Ping")))
			replyToPing();
		else if ((r.containsKey("sysRequest")) &&
			(r.get("sysRequest").equals("Get Servlet Run Status")))
			getServletRunStatus();
		else
			throw new ThrowableString("unrecognised request >" + (String) r.get("sysRequest") +
					"< or missing input parameters");
	}


	//-----------------------------------------------------------
	public void replyToPing() throws ThrowableString
	{
		//if (r.containsKey("sysPing"))
			r.put("sysPing", "Live");
			r.put("sysRepyingInstance", svr.servletName);
			r.put("sysRepyingHost", svr.servletHostName);
	}

	//-----------------------------------------------------------
	public void getServletRunStatus() throws ThrowableString
	{
                if (svr.servletState == RunStatusTyp.running)
                        r.put("sysRunStatus", "Running");
                else if (svr.servletState == RunStatusTyp.halted)
                        r.put("sysRunStatus", "Halted");
                else
                        r.put("sysRunStatus", "!Undefined!");
	}

	//-----------------------------------------------------------
	public void getConfigValues() throws ThrowableString
	{
	 int i = 0;
	 String key = "";
	 String buf = "";

		for (Enumeration it = svr.configParams.keys(); it.hasMoreElements();)
		{
			key = "sysConfigParamName" + Integer.toString(i);
			buf = (String) it.nextElement();
			r.put(key, buf);				// parameter name
			key = new String("sysConfigParamValue" + Integer.toString(i++));
									// parameter value
			r.put(key, (String) ((ServletParamInfo) svr.configParams.get(buf)).value);
		}
		r.put("Requests Received", Integer.toString(svr.requestsReceived));

		getServletRunStatus();
	}

	//-----------------------------------------------------------
	public void executeMethod() throws ThrowableString
	{
	 Enumeration it = null;

		String Rmethod = (String) r.get("sysMethod");
		if (Rmethod.equals("BeginTransaction"))
		{
			if (!r.containsKey("sysTransaction"))
				throw new ThrowableString("Missing sysTransaction in input data");

			loadTid(svr.transactionList);

			if (svr.transactionList.contains((String)r.get("sysTransaction")))
				throw new ThrowableString("Transaction with this same id " + (String) r.get("sysTransaction") +
							" already in progress cannot begin again!");
			svr.beginTransaction(r);

			svr.transactionList.add((String) r.get("sysTransaction"));

			saveTid(svr.transactionList);

			return;
		}
		else if (Rmethod.equals("CommitTransaction"))
		{

			if (!r.containsKey("sysTransaction"))
				throw new ThrowableString("Missing sysTransaction in input data");

			loadTid(svr.transactionList);

			if (svr.transactionList.contains((String) r.get("sysTransaction")))
				throw  new ThrowableString("Transaction with id " + (String) r.get("sysTransaction") +
							" does not exist, cannot commit!");
			svr.commitTransaction(r);
			svr.transactionList.remove((String) r.get("sysTransaction"));

			saveTid(svr.transactionList);

			return;
		}
		else if (Rmethod.equals("RollBackTransaction"))
		{
			if (!r.containsKey("sysTransaction"))
				throw new ThrowableString("Missing sysTransaction in input data");

			loadTid(svr.transactionList);

			if (!svr.transactionList.contains((String) r.get("sysTransaction")))
				throw new ThrowableString("Transaction with id " + (String) r.get("sysTransaction") +
							" does not exist, cannot roll back!");
			svr.rollBackTransaction(r);
			svr.transactionList.remove((String) r.get("sysTransaction"));

			saveTid(svr.transactionList);

			return;
		}

		if (!r.containsKey("sysObject"))
			throw new ThrowableString("Missing sysObject in input data");

		String Robj = (String) r.get("sysObject");
		if (!svr.objs.containsKey(Robj))
			throw new ThrowableString("unknown object requested: " + Robj);

		ServletMethodTyp m = ((ServletObjectInfo) svr.objs.get(Robj)).methods;
		if (!m.containsKey(Rmethod))
			throw new ThrowableString("unknown method requested: " + Rmethod);

		System.err.println("Calling: " + svr.assignedName + "." + Robj + "." + Rmethod);

		((ServletObj) m.get(Rmethod)).run(r);
	}

	//-----------------------------------------------------------
	// get the connector specifications; objects, 
	// fields, methods and parameter configurations
	public void servletSpec() throws ThrowableString
	{
	 int i = 0;
	 int j = 0;
	 String okey = "";
	 String fkey = "";
	 String mkey = "";
	 String key = "";
	 String servletName = "";

		if (!r.containsKey("sysServletName"))
		{
			String errMsg = new String("ERR: servletSpec() - Missing sysServletName parameter in input");
			//System.err.println(errMsg);
			throw new ThrowableString(errMsg);
		}
		servletName = (String) r.get("sysServletName");

		ServletObjTyp objs = svr.objs;
		//r.put("sysHostName", svr.servletHostName);
											// for every object
		for (Enumeration oit = objs.keys(); oit.hasMoreElements();)
		{
			okey = (String) oit.nextElement();
			ObjFieldTyp fields = ((ServletObjectInfo) objs.get(okey)).fields;
											// for every field
			for (Enumeration fit = fields.keys(); fit.hasMoreElements();)
			{
				fkey = (String) fit.nextElement();
				key = "Field" + Integer.toString(i++);
				r.put(key, svr.servletHostName + ":" + servletName + "." + okey + "." + fkey);
			}
			ServletMethodTyp methods = ((ServletObjectInfo) objs.get(okey)).methods;
											// for every method
			for (Enumeration mit = methods.keys(); mit.hasMoreElements();)
			{
				mkey = (String) mit.nextElement();
				key = "Method" + Integer.toString(j++);
				r.put(key, svr.servletHostName + ":" + servletName + "." + okey + "." + mkey);
			}
		}
										// Configurable parameters for this connector
										// Configurable parameters for this connector
		i = 0;
		String paramName = "";
		String element = "";
		String tmpBuf = "";
		String sysParam = "";
		for (Enumeration it = svr.configParams.keys(); it.hasMoreElements();)
		{
									// paramater name
			paramName = (String) it.nextElement();
			key = "sysConfigParamName" + Integer.toString(i);
			r.put(key, paramName);
									// paramater options
			Vector options = ((ServletParamInfo) svr.configParams.get(paramName)).opt;
			j = 0;
			sysParam = "sysParam" + Integer.toString(i);
			for (Enumeration oit = options.elements(); oit.hasMoreElements();)
			{
				element = (String) oit.nextElement();
				tmpBuf = sysParam + "Opt" + Integer.toString(j++);
				r.put(tmpBuf, element);
			}
									// parameter size/length
			key = "sysParamSize" + Integer.toString(i);
			tmpBuf = Integer.toString(((ServletParamInfo) svr.configParams.get(paramName)).size);
			r.put(key, tmpBuf);
									// paramater input allowed
			key = "sysParamAllow" + Integer.toString(i);
			tmpBuf = ((ServletParamInfo) svr.configParams.get(paramName)).allow;
			r.put(key, tmpBuf);
									// display parameter True/False
			key = "sysParamDisplay" + Integer.toString(i++);
			if (((ServletParamInfo) svr.configParams.get(paramName)).display)
				r.put(key, "True");
			else
				r.put(key, "False");
		}
	}

	//-----------------------------------------------------------
	public void saveTid(TransactionListTyp tl) throws ThrowableString
	{
		// Serialize access to tid file
		tidFileSem.getSem();

		String fn = System.getProperty("M_WARE_TID") + "/" + svr.assignedName + ".tid";
		PersistStream ps = new PersistStream(fn);
		ps.write(svr.transactionList);
		tidFileSem.releaseSem();
	}

	//-----------------------------------------------------------
	public void loadTid(TransactionListTyp tl) throws ThrowableString
	{
		// Serialize access to tid file
		tidFileSem.getSem();

		String fn = System.getProperty("M_WARE_TID") + "/" + svr.assignedName + ".tid";
		File aFile = new File(fn);
		if (aFile.canRead())
		{
			PersistStream ps = new PersistStream(fn);
			tl.clear();
			ps.write(tl);
		}
		tidFileSem.releaseSem();
	}

	//-----------------------------------------------------------
	public void run()
	{
		System.err.println("------------------------------------------------");
		System.err.println("RequestExecuter::run() - Received request");
		dt = new DateTime();
		System.err.println(dt.str());
		System.err.println(svr.procName + "-> DATA RECEIVED FOR REQUEST TRANSACTION:");
		PersistStream.printMap(r);

		boolean doReply = true;		// defaults on true in in the case where sysReply is missing
		if (r.containsKey("sysReply") && (((String) r.get("sysReply")).equals("False")))
			doReply = false;
		try {
			processMsg();
		} catch (ThrowableString e) {
			String errMsg = new String("ERR: RequestExecuter::processMsg() - " + e);
			System.err.println(errMsg);
			r.put("sysResultString", errMsg);
			r.put("sysResultStatus", "-1");
		}

		dt = new DateTime();
		System.err.println(dt.str());
		if (doReply)
		{
			System.err.println("------------------------------------------------");
			System.err.println("Replying");
			System.err.println(svr.procName + "-> OUTPUT DATA RETURNED TO CALLER:  ");
			PersistStream.printMap(r);

			try {
				sok.write(r);
				sok.close();
			} catch (ThrowableString e) {
				System.err.println(e);
			}
		}
		else
			System.err.println("Done... no reply requested");
	}           
}

