
//-----------------------------------------------------------
//
//	Softbolts Ltd. (c) all rights reserved
//
//	Warning: This  program is  protected by copyright law 
//		 and   international  treaties.  Unauthorised
//		 reproduction or distribution of this program,
//		 or  any portion  or it, may result in severe
//		 civil  and  criminal  penalties, and will be
//		 prosecuted to  the  maximum extent possible.
//
//              Author: A. E. Dawson, Oct 2002 
//              mailto: anthony@softbolts.com
//
//		Last Revised: 14 Oct 2002
//
//-----------------------------------------------------------

package com.softbolts.lib;

import java.io.*;
import java.net.*;
import java.util.*;
import java.text.*;
import java.lang.*;

//-----------------------------------------------------------
public class ServletRunner
{
 static boolean runStatus;
 private static final String pidExt = ".pid";
 private static final String defaultLogDir = "/tmp";
 private static int ourPid;
 private boolean callInit = false;
 private QSemaphore tidFileSem = null;
 private static SharedBool pollerState = null;
 private static Thread cth = null;
 private BaseServlet svr = null;
 private NameValueTyp r = null;
 private DateTime dt = null;
 private boolean redirectStdErr = true;

        public ServletRunner(BaseServlet svr, String args[]) throws ThrowableString
        {
	 String requiredName = null;

		this.svr = svr;
		if (args.length <= 0)
			this.svr.procName = "No Name!";
		else 
			this.svr.procName = args[0];

                if (args.length >= 1)
                        requiredName = args[1];
		else
			requiredName = "";

		if ((args.length >= 3) && (args[2].equals("-noredirect")))
			redirectStdErr = false;
		else
			redirectStdErr = true;

		tidFileSem = new QSemaphore(1);
		pollerState = new SharedBool(false);
		r = new NameValueTyp();
		cth = Thread.currentThread();
/*
		if (cth.isDaemon() == true)
			System.err.println("Process is Daemon");
		else
			System.err.println("Process is NOT Daemon");

		if (System.getProperty("M_WARE_TMP") == null)
		{
			throw new ThrowableString(svr.procName + " ERR: M_WARE_TMP is not set!");
		}
*/

		if (System.getProperty("M_WARE_TID") == null)
		{
			throw new ThrowableString(svr.procName + " ERR: M_WARE_TID is not set!");
		}

		if (System.getProperty("M_WARE_LOGS") == null)
		{
			throw new ThrowableString(svr.procName + " ERR: M_WARE_LOGS is not set!");
		}

/*
		if (System.getProperty("M_WARE_PATH") == null)
		{
			throw new ThrowableString(svr.procName + " ERR: M_WARE_PATH is not set!");
		}
*/

		if (System.getProperty("M_WARE_CONFIGDIR") == null)
		{
			throw new ThrowableString(svr.procName + " ERR: M_WARE_CONFIGDIR is not set!");
		}

		if (System.getProperty("M_WARE_ENGINEHOSTNAME") == null)
		{
			throw new ThrowableString(svr.procName + " ERR: M_WARE_ENGINEHOSTNAME env variable not set");
		}

		if (System.getProperty("M_WARE_ENGINESOCKET") == null)
		{
			throw new ThrowableString(svr.procName + " ERR: M_WARE_ENGINESOCKET env variable not set");
		}

		if ((svr.procName == null) || (svr.procName.equals("")))
		{
			throw new ThrowableString("ERR: No Connector name");
		}

		svr.engineHostName = System.getProperty("M_WARE_ENGINEHOSTNAME");
		String port = System.getProperty("M_WARE_ENGINESOCKET");
		
		if (!Character.isDigit(port.charAt(0)))
		{
			throw new ThrowableString("ERR: environment variable M_WARE_ENGINESOCKET incorrectly set!");
		}

		//svr.enginePort = new Integer(port).intValue();
		//svr.enginePort = Integer(port).intValue();
		svr.enginePort = Integer.parseInt(port);

		runStatus = true;

		try {
			Register p = new Register(svr, requiredName);

		} catch (ThrowableString s) {
			throw new ThrowableString("ERR: main() - Register() threw up! -> " + s.msg);
		}

		try {
			if (svr.servletState == RunStatusTyp.running)
				svr.init(r);
		} catch (ThrowableString s) {
			svr.servletState = RunStatusTyp.halted;
			// if we throw up here the connector dies and that is the end 
			// we have no chnce to restart it though the runtime monitor
			// and that is inconsistent with the other call to init
			//throw new ThrowableString("ERR: Servlet::init() threw up! -> " + s.msg);
		}

		String pidFn = svr.assignedName + pidExt;

		// detach process from shell
		// I might be wrong but I don't think this has
		// much meaning in java... Well someone tells me I could use Process!

		if (redirectStdErr == true)
		{
			System.out.println("--------------------------------------------------");
			System.out.println("--------------------------------------------------");
			System.out.println("NOTE: For servlet logs, run the following command:");
			if ((System.getProperty("M_WARE_LOGS") != null) && (!System.getProperty("M_WARE_LOGS").equals("")))
				System.out.print("\"touch " + System.getProperty("M_WARE_LOGS"));
			else
				System.out.print("\"touch " + defaultLogDir);
			System.out.println("/" + svr.assignedName + ".log" + "\"");
			System.out.println("--------------------------------------------------");

			String fn =  System.getProperty("M_WARE_LOGS") + "/" + svr.assignedName + ".log";

			boolean exists = new File(fn).exists();
			if (exists == false)
			{
				System.out.println("--------------------------------------");
				System.out.println("**NOTE**");
				System.out.println("Can't locate log file for servlet " + svr.procName);
				System.out.println(fn + "\nor file has wrong permissions");
				System.out.println("All errors will be redirected to stderr");
				System.out.println("--------------------------------------");
			}
			else
			{
				try {
					System.err.println("Setting error file to: " + fn); 
					FileOutputStream out = new FileOutputStream(fn, true);
					PrintStream ps = new PrintStream(out);
					System.setErr(ps);
				} catch (FileNotFoundException fnf) {
					System.err.println("ServletMain() - file not found exception " + fnf.getMessage()); 
				}
			}
			System.err.println("--------------------------------------------------");
		}
		dt = new DateTime();
		System.err.println(svr.procName + " Daemon started: " + dt.str());
		//System.err.println("PID = " << getpid() << endl;
	}

	//-----------------------------------------------------------
        public void run() throws ThrowableString
	{
		SocketObj sok = null;
		try {
			sok = new SocketObj(svr.ourPort);
		} catch (ThrowableString s) {
			throw new ThrowableString(s.msg);
		}

		//svr.servletState = RunStatusTyp.running;

		boolean doReply = true;
		while (runStatus)
		{
			r.clear();
			System.err.println("----------------------");
			System.err.println("Waiting for a request! on port: " + svr.ourPort);
			try {
				sok.getConnection();
				svr.requestsReceived++;
				sok.read(r);
				if (r.containsKey("sysClientId") == false)
				{
					r.put("sysResultStatus", "-1");
					r.put("sysResultString", 
						"ERR: No sysTransId parameter found in request or missing sysClientId");
					sok.write(r);
					sok.close();
					continue;
				}

				doReply = true;
				if (r.containsKey("sysReply") && (((String) r.get("sysReply")).equals("False")))
				{
					r.put("sysResultStatus", "0");
					r.put("sysResultString", "OK");
					sok.write(r);
					sok.close();
					doReply = false;
				}

			} catch (ThrowableString s) {
				throw new ThrowableString(s.msg);
			}

System.err.println("---------RECEIVED REQUEST----------------------");
PersistStream.printMap(r);
System.err.println("-----------------------------------------------");
			if ((r.containsKey("sysRequest") == true) && ((((String) r.get("sysRequest")).equals("Set Param"))  ||
				(r.get("sysRequest").equals("Tmp Servlet Conf")) ||
				(r.get("sysRequest").equals("Set Run State")) ||
				(r.get("sysRequest").equals("Terminate")) ||
				(r.get("sysRequest").equals("Pinger Update"))))
			{   	                                      // Routine that alter configuration information though here
				if (((String) r.get("sysRequest")).equals("Terminate"))   // Special case: request to 
				{							  // Terminate the connector
	System.err.println(" RECEIVED TERMINATE REQUEST");
					runStatus = false;
					pollerState.clear();
					try {
						svr.finish(r);
					} catch (ThrowableString s) {
						throw new ThrowableString(s.msg);
					}
					r.put("sysResultStatus", "0");
				}
				else
				{
					try {
System.err.println("Doing sysRequest");
						sysRequest();
						r.put("sysResultStatus", "0");
						System.err.println( "sysRequest Done OK");
					} catch (ThrowableString s) {
						String tmp = new String("ERR: sysRequest() - " + s.msg);
						System.err.println(tmp);
						r.put("sysResultString", tmp);
						r.put("sysResultStatus", "-1");
					}
				}

				if (doReply)
				{
					System.err.println("-------------------------------");
					System.err.println("Replying with: ");
					PersistStream.printMap(r);
					try {
						sok.write(r);
						sok.close();
					} catch (ThrowableString s) {
						throw new ThrowableString(s.msg);
					}

					System.err.println("Reply DONE");
				}
			}	// the next line needs to be commented out
				// or the rutime console cannot request connector details etc
			else //if (svr.servletState == RunStatusTyp.running)	// Routines that just refer to 
			{							// configuraton info through here
				if (svr.procMode == ProcModeTyp.multi_threaded)
				{
					try {	// runs as a separate thread
						System.err.println("... Running in separate thread");
						ServletRunnerThread runner = new ServletRunnerThread(tidFileSem, sok.getSocket(), svr, r);
						runner.start();
					} catch (ThrowableString s) {
						System.err.println(s.msg);
					}
					System.err.println("... Back in main thread");
				}
				else if (svr.procMode == ProcModeTyp.single_threaded)
				{
					System.err.println("... Running as same thread");
					RequestExecuter runner = new RequestExecuter(tidFileSem, sok.getSocket(), svr, r);
					runner.run();
				}
				else
					System.err.println("ERR: svr.procMode is not set");

				// dt = new DateTime();
				// System.err.println("Transaction completed." + dt.str());
				// System.err.println("-------------------------------");
				// return;
			}
		}

		// exits though here if Terminate message is received.
		pollerState.clear();
		try {
			sok.closeListener();
			cth.sleep(1000);
		} catch (ThrowableString s) {
			System.err.println(s.msg);
		} catch (InterruptedException ie) {
			System.err.println("ERR: unexpected interrupt exception while sleeping -> " + ie.getMessage());
		}

		dt = new DateTime();
		System.err.println("Stopping Servlet " + svr.assignedName + " " + dt.str());
		System.err.println("Stopped....");
		System.err.println("-------------------------------");
		//unlink(pidFn.c_str());
        }

	//-----------------------------------------------------------
	public void sysRequest() throws ThrowableString
	{

		if (!r.containsKey("sysClientId"))
			throw new ThrowableString("No sysClientId found in request");

		if (((String) r.get("sysRequest")).equals("Set Param"))
		{
									// required: sysParamsName, sysParamValue
			if (!r.containsKey("sysParamName"))
				throw new ThrowableString("No sysParamName found in request");

			if (!r.containsKey("sysParamValue"))
				throw new ThrowableString("No sysParamValue found in request");

			if (!svr.configParams.containsKey(r.get("sysParamName")))
				throw new ThrowableString(r.get("sysParamName") + " is not a configurable parameter");

			callInit = (!(svr.configParams.get(r.get("sysParamName"))).value.equals((r.get("sysParamValue"))));

	System.err.println("Saving: " + (String) r.get("sysParamName"));
	System.err.println("Value: " + (String) r.get("sysParamValue"));
			svr.configParams.get(r.get("sysParamName")).set = true;
			svr.configParams.get(r.get("sysParamName")).value = r.get("sysParamValue");

			String fn = System.getProperty("M_WARE_CONFIGDIR") + "/" + svr.assignedName + ".cfg";
			PersistStream ps = new PersistStream(fn);
			ps.write(svr.configParams);

			if (callInit)
			{
				if (svr.servletState == RunStatusTyp.running)
				{
					r.put("sysRefresh", "True");
					svr.init(r);
				}
			}
			if (!r.containsKey("sysOpResult")) r.put("sysOpResult", "");
		}
		else if ((r.get("sysRequest")).equals("Set Run State"))
		{
			if (!r.containsKey("sysState"))
				throw new ThrowableString("No sysState found in data for command Set Run State");

			if ((r.get("sysState")).equals("Run"))
			{
				svr.servletState = RunStatusTyp.running;
				r.put("sysService", "Start Servlet");
				svr.service(r);
				svr.init(r);
			}
			else
			{
				svr.servletState = RunStatusTyp.halted;
				r.put("sysService", "Halt Servlet");
				svr.service(r);
				pollerState.clear();
			}
		}
		else if (r.get("sysRequest").equals("Tmp Servlet Conf"))
		{
	System.out.println("ServletRunner::sysRequest() - Tmp Servlet Conf");

			for (int i = 0; r.containsKey("sysParamName" + Integer.toString(i)); i++)
			{
				if (!r.containsKey("sysParamValue" + Integer.toString(i)))
					throw new ThrowableString("No sysParamValue found in request");

				if (!svr.configParams.containsKey(r.get("sysParamName" + Integer.toString(i))))
					throw new ThrowableString(r.get("sysParamName" + Integer.toString(i)) + 
							" is not a configurable parameter");

				svr.configParams.get(r.get("sysParamName" + Integer.toString(i))).set = true;
				svr.configParams.get(r.get("sysParamName" + Integer.toString(i))).value = 
									r.get("sysParamValue" + Integer.toString(i));
			}

			RunStatusTyp oldState = svr.servletState;
			svr.servletState = RunStatusTyp.running;
			svr.init(r);
			NameValueTyp tmpMap = r;
			svr.finish(r);
			svr.servletState = oldState;
		}
		else if (r.get("sysRequest").equals("Pinger Update"))
		{
			svr.service(r);
		}
	}
}

