
//-----------------------------------------------------------
//
//	Softbolts Ltd. (c) all rights reserved
//
//	Warning: This  program is  protected by copyright law 
//		 and   international  treaties.  Unauthorised
//		 reproduction or distribution of this program,
//		 or  any portion  or it, may result in severe
//		 civil  and  criminal  penalties, and will be
//		 prosecuted to  the  maximum extent possible.
//
//              Author: A. E. Dawson, Aug 1999
//              mailto: anthony@softbolts.com
//
//		Last Revised: 3 July 2002
//
//-----------------------------------------------------------

package com.softbolts.lib;

import java.io.*;
import java.net.*;
import java.util.*;
import java.lang.*;

public class SocketObj
{
					/** indicates the state of the socket */
 public  boolean connected = false;
 private int port;
 private String hostName = null;
 private Socket sok = null;
 private DataInputStream in = null;
 private DataOutputStream out = null;
 private String key, value;
 public final static String EOT = "\004";
 public final static String CR = "\r";
 public final static String LF = "\n";
 public final static String CRLF = "\r\n";
 private static final Thread cth = Thread.currentThread();
 private ServerSocket ssok = null;
 private SocketTyp type = SocketTyp.undefined;
 private ProtocolTyp protocol = ProtocolTyp.thunderbolt;

	//-----------------------------------------------------------
	/**
	* Returns the under lying Socket.
	*
	*/
	public Socket getSocket()
	{
		return sok;
	}

	//-----------------------------------------------------------
	/**
	* Creates a socket object with a given Socket
	*
	* @param sok The socket to use in the creation of the SocketObj.
	*/
	public SocketObj(Socket sok) throws ThrowableString
	{
		this.sok = sok;
		try {
			in = new DataInputStream(sok.getInputStream());
			out = new DataOutputStream(sok.getOutputStream());
		} catch (IOException e) {
			throw new ThrowableString("SocketObj::SocketObj() - " + e.getMessage());
		}
		type = SocketTyp.server;
	}

	//-----------------------------------------------------------
	/**
	* Opens a connection towards the given host on the given port.
	*
	* @param port The port to connect to.
	* @param hostName The host name to connect to.
	*/
	public SocketObj(int port, String hostName) throws ThrowableString, IOException
	{
	 byte[] ipAddr;

//System.err.println("SocketObj()");
//System.err.println("port: " + port);
//System.err.println("hostName: " + hostName);
		connected = false;

		this.port = port;
		if (Character.isDigit(hostName.charAt(0)))		// we need the name not the ip
		{
			try {
				InetAddress addr = InetAddress.getByName(hostName);

				// Get IP Address
				ipAddr = addr.getAddress();

				// Get hostname
				hostName = addr.getHostName();
			} catch (UnknownHostException e) {
				System.err.println("BaseServlet() - getHostName() - ");
			}
		}

		this.hostName = hostName;
		try {
			sok = new Socket(hostName, port);
			//sok.setTcpNoDelay(true);					// reset timeout
			//sok.setSendBufferSize(1);					// reset timeout
			in = new DataInputStream(sok.getInputStream());
			out = new DataOutputStream(sok.getOutputStream());
		} catch (UnknownHostException he) {
			throw new ThrowableString("SocketObj() - Unknow host: " + hostName + " " + he.getMessage());
		}
		//} catch (IOException ioe) {
			//throw new ThrowableString("SocketObj() - IO exception: " + ioe.getMessage());
		//}
		connected = true;
		type = SocketTyp.client;
	}           

	//-----------------------------------------------------------
	public void setPtotocol(ProtocolTyp protocol)
	{
		this.protocol = protocol;
	}

	//-----------------------------------------------------------
	/**
	* Waits to receive an incomming connection.
	*
	*/
	public void getConnection() throws ThrowableString
	{
		if (type != SocketTyp.server)
			throw new ThrowableString("getConnection() - can only be called on a server socket type");
		try {
			sok = ssok.accept();
			sok.setTcpNoDelay(true);
			in = new DataInputStream(sok.getInputStream());
			out = new DataOutputStream(sok.getOutputStream());
		} catch (IOException e) {
			throw new ThrowableString(e.getMessage());
		}
		connected = true;
	}

	//-----------------------------------------------------------
	/**
	* Creates a server socket on the given port.
	*
	* @param port The port number on which to create the service.
	*/
	public SocketObj(int port) throws ThrowableString
	{
		this.port = port;
		try {
			ssok = new ServerSocket(port);
		} catch (UnknownHostException he) {
			throw new ThrowableString("SocketObj() - Unknow host: " + hostName + " " + he.getMessage());
		} catch (IOException ioe) {
			throw new ThrowableString("SocketObj() - IO exception: " + ioe.getMessage() + " for port: " + port);
		}
		type = SocketTyp.server;
	}

//HERE

	//-----------------------------------------------------------
	/**
	* Reads in a Vector
	*
	* @param data The holder in which the data will be placed.
	*/
	public void read(Vector data) throws ThrowableString
	{
	 byte buff[] = new byte[1];
	 String line = "";
	 int c = 1;
	 int r = 0;

		data.clear();
		try {
			while ((line.equals(CRLF) == false) && (r != -1))
			{
				line = "";
				buff[0] = 1;
				while (buff[0] != 0 && (r != -1))
				{
					// setting the timeout causes the socket to time out imediatelly under
					// windows 2K and XP but not under Linux so much for java portability!
					//sok.setSoTimeout(5);		// set timeout
					if ((r = in.read(buff, 0, 1)) == 0) continue;
					if (buff[0] != 0)
						line = line + Character.toString((char) buff[0]);

					if (line.lastIndexOf(CRLF) != -1) break;
				}

				if (line.equals(CRLF) == false) data.add(line);

			}
			//sok.setSoTimeout(0);		// reset timeout
		} catch (SocketException se) {
			throw new ThrowableString("Socket exception " + se.getMessage());
		} catch (IOException ioe) {
			throw new ThrowableString("SocketObj::read() - IO Socket exception " + ioe.getMessage());
		}
			
/*
		Enumeration e = v.elements();
		while (e.hasMoreElements())
		{
			data.put((String) e.nextElement(), (String) e.nextElement());
		}
*/
	}

	//-----------------------------------------------------------
	/**
	* Reads in a NameValueTyp
	*
	* @param data The holder in which the data will be placed.
	*/
	public void read(NameValueTyp data) throws ThrowableString
	{
	 byte buff[] = new byte[1];
	 String str = "";
	 Vector v = new Vector(100);
	 int c = 1;
	 int r = 0;

		v.clear();
		try {
			while (str.equals(EOT) == false)
			{
				str = "";
				buff[0] = 1;
				while (buff[0] != 0) // && in.available() > 0) || r == -1)
				{
					// setting the timeout causes the socket to otime out imediatelly under
					// windows 2K and XP but not under Linux so much for java portability!
					//sok.setSoTimeout(5);		// set timeout
					if ((r = in.read(buff, 0, 1)) == 0) continue;
					if (buff[0] != 0)
						str = str + Character.toString((char) buff[0]);
				}
				if (str.equals(EOT) == false)
				{
					v.add(str);
				}

			}
			sok.setSoTimeout(0);		// reset timeout
		} catch (SocketException se) {
			throw new ThrowableString("Socket exception " + se.getMessage());
		} catch (IOException ioe) {
			throw new ThrowableString("SocketObj::read() - IO Socket exception " + ioe.getMessage());
		}
		if (v.size() > 0)
		{
			if ((v.size() % 2) != 0)
				throw new ThrowableString("Socket() - Name/value param mismatch read in: " + v.size() + " prams.");
			
			Enumeration e = v.elements();
			while (e.hasMoreElements())
			{
				data.put((String) e.nextElement(), (String) e.nextElement());
			}
		}
	}

	//-----------------------------------------------------------
	/**
	* Writes out a String though the socket using writeChars().
	* 
	* @param str The wtring to be written.
	*/
	public void write(String str) throws ThrowableString
	{
		try {
			out.writeChars(str);
			out.flush();
		} catch (IOException ioe) {
			throw new ThrowableString("SocketObj::write(String str) - IO exception" + ioe.getMessage());
		}
	}

	//-----------------------------------------------------------
	/**
	* Writes out an HTML header. Not currently used.
	*
	*/
	public void writeHeader() throws ThrowableString
	{
	 String header = "";

		header = "HTTP/1.0 200 OK\r\n" + 
			 "Date: Fri, 31 Dec 1999 23:59:59 GMT\r\n" +
			 "Server: Thunderbolt HTTP connector V1.1.2\r\n" +
			 "Content-Type: text/html\r\n" + 
			 "Allow: GET\r\n" +
			 "Connection: close\r\n";
//"Allow: GET, HEAD, OPTIONS, TRACE" +
//Content-Length: 1354

		try {
			out.writeChars(header);
			out.flush();
		} catch (IOException ioe) {
			throw new ThrowableString("SocketObj::writeChars(String s) - IO exception" + ioe.getMessage());
		}
	} 

	//-----------------------------------------------------------
	/**
	* Writes out the contents of a file in binary form.
	*
	* @param fn The file name inclusive of path.
	*/
	public void writeBinaryFile(String fn) throws ThrowableString
	{
	 File f = new File(fn);
	 int c;
	 byte buffer[] = null;

		if (f.canRead() == false)
		{
			try{
				sok.close();
			} catch (IOException ioe) {
				System.err.println("1) IOException when trying to close the " +
						   "socket: " + ioe.getMessage());
			}
			throw new ThrowableString("SocketObj::writeBinaryFile(String fn) - can't read: " + fn);
		}

		//InputStream is = fr.getInputStream();
/*
		FileReader fr = null;
		int got = 0;
		char ch;
		try {
			fr = new FileReader(fn);
			//while ((got = fr.read(ch)) >= 0) 
			while ((ch = fr.read()) >= 0) 
			{
				//if (got == 0) continue;

				out.write(ch);
			}
		} catch (FileNotFoundException fnf) {
			throw new ThrowableString(fnf.getMessage());
		} catch (IOException ioe) {
			throw new ThrowableString("SocketObj::writeBinaryFile(String fn) - IOException: " + ioe.getMessage());
		}
*/

// TO DO
//		if (protocol == ProtocolTyp.http)
//			writeHeader(fn);
		FileReader fr = null;
		try {
			fr = new FileReader(fn);

			while ((c = fr.read()) != -1)
				//out.write(Integer(c).byteValue());
				out.write(c);

			fr.close();
		} catch (FileNotFoundException fnf) {
			try {
				fr.close();
				sok.close();
			} catch (IOException ioe) {
				System.err.println("1) IOException when trying to close file and or " +
						   "socket after already having thrown an FileNotFoundException: " + 
						    ioe.getMessage() + " The original exception: " + fnf.getMessage());
			}
			throw new ThrowableString(fnf.getMessage());
		} catch (IOException ioe) {
			try {
				fr.close();
				sok.close();
			} catch (IOException ioe2) {
				System.err.println("2) IOException when trying to close file and or " +
						   "socket after already having thrown an IOException: " + 
						    ioe2.getMessage() + " The original exception: " + ioe.getMessage());
			}
			throw new ThrowableString("SocketObj::writeBinaryFile(String fn) - IOException: " + ioe.getMessage());
		}
	}

	//-----------------------------------------------------------
	/**
	* Writes out a byte array.
	*
	* @param data The byte array to write out.
	*/
	public void write(byte[] data) throws ThrowableString
	{
		try {
			out.write(data, 0, data.length) ;
			out.flush() ;

		} catch (FileNotFoundException fnf) {
			throw new ThrowableString(fnf.getMessage());
		} catch (IOException ioe) {
			throw new ThrowableString("SocketObj::write(int data) - IO exception" + ioe.getMessage());
		}
	}

	//-----------------------------------------------------------
	/**
	* Writes out an int
	*
	* @param data The int to write out.
	*/
	public void write(int data) throws ThrowableString
	{
		try {
			out.write((char) data);
		} catch (FileNotFoundException fnf) {
			throw new ThrowableString(fnf.getMessage());
		} catch (IOException ioe) {
			throw new ThrowableString("SocketObj::write(int data) - IO exception" + ioe.getMessage());
		}
	}

	//-----------------------------------------------------------
	/**
	* Writes out an NameValueTyp.
	*
	* @param data The NameValueTyp to write out.
	*/
	public void write(NameValueTyp data) throws ThrowableString
	{

		if (data.containsKey("sysTimeout") == false)
			data.put("sysTimeout", "5");
		try {
			Enumeration it = data.keys();
			sok.setSoTimeout(5);					// set timeout
			while (it.hasMoreElements())
			{
				key = (String) it.nextElement();
				value = (String) data.get(key);
				out.writeBytes(key);
				out.write(0);
				out.writeBytes(value);
				out.write(0);
			}
			out.writeBytes(EOT);					// announce EOT
			out.write(0);
			out.flush();
			sok.setSoTimeout(0);					// reset timeout
		} catch (SocketException se) {
			throw new ThrowableString("Socket exception " + se.getMessage());
		} catch (IOException ioe) {
			throw new ThrowableString("IO Socket exception " + ioe.getMessage());
		}
	}

	//-----------------------------------------------------------
	public void open() throws ThrowableString
	{
	}

	//-----------------------------------------------------------
	public void closeListener() throws ThrowableString
	{
	}

	//-----------------------------------------------------------
	/**
	* Closes the socket.
	*
	*/
	public void close() throws ThrowableString
	{
		try {
			in.close();
			out.close();
			sok.close();
		} catch (IOException ioe) {
			throw new ThrowableString("Socket IO exception while closing " + ioe.getMessage());
		}
		connected = false;
	}
}


