
//-----------------------------------------------------------
//
//	Softbolts Ltd. (c) all rights reserved
//
//	Warning: This  program is  protected by copyright law 
//		 and   international  treaties.  Unauthorised
//		 reproduction or distribution of this program,
//		 or  any portion  or it, may result in severe
//		 civil  and  criminal  penalties, and will be
//		 prosecuted to  the  maximum extent possible.
//
//              Author: A. E. Dawson, Oct 2002 
//              mailto: anthony@softbolts.com
//
//		Last Revised: 14 Oct 2002
//
//-----------------------------------------------------------

package com.softbolts.lib;

import java.io.*;
import java.net.*;
import java.util.*;
import java.text.*;
import java.lang.*;

//-----------------------------------------------------------
/**
* Contains generic utilities used to construct servlets
*
* @author Anthony E. Dawson mail: anthony@softbolts.com
*
*/
public class Utils
{
	/**
	* Makes a request to the engine to execute the transaction 
	* model passed as a parameter.
	* @exception TrowableString If there is a failure in the request.
	* @param svr The servlet being run.
	* @param model The model to be run by the engine.
	* @param sync When set it requests the model to be run syncrounously.
	* @param debug Runs with full debugging on when set.
	*/
        public static NameValueTyp runModel(NameValueTyp tmpMap, BaseServlet svr, String model, 
						boolean sync, boolean debug) throws ThrowableString
        {
	 SocketObj sok = null;

		tmpMap.put("sysTransName", model);
		tmpMap.put("sysClientId", svr.assignedName);
		if (debug) tmpMap.put("sysDebug", "True");
		else tmpMap.put("sysDebug", "False");
		if (sync) tmpMap.put("sysReply", "True");
		else tmpMap.put("sysReply", "False");

		//PersistStream.printMap(tmpMap);
		try {
			sok = new SocketObj(svr.enginePort, svr.engineHostName);
			if (sok.connected)
			{
				sok.write(tmpMap);
				sok.read(tmpMap);
				sok.close();
			}
		        else // (!sok.connected)
			{
				tmpMap.put("sysResultStatus", "-1");
				tmpMap.put("sysResultString", "Socket Connection failed");
			}

		} catch (IOException ioe) {
			throw new ThrowableString(ioe.getMessage());
		}

		return tmpMap;
	}

	//-----------------------------------------------------------
	/**
	* Verifies the string to be the name of a valid transaction model
	* rgistered with the engine. On success it returns the data model 
	* name on failure it throws a TrowableString exception.
	* @exception TrowableString If the model doesn't verify
	* @param svr The servlet being run.
	* @param model The model name to be verified.
	*/
        public static String verifyModel(BaseServlet svr, String model) throws ThrowableString
        {
	 boolean validModel = false;

                if (model.equals("") == false)
                {

                        NameValueTyp tmpMap = new NameValueTyp();
                        tmpMap.put("sysRequest", "List Models");
                        tmpMap.put("sysClientId", svr.assignedName);
                        tmpMap.put("sysDebug", "True");
                        tmpMap.put("sysReply", "True");

                        SocketObj sok = null;
                        try {
                                sok = new SocketObj(svr.enginePort, svr.engineHostName);
                        } catch (IOException ioe) {
                                System.err.println("Utils::verifyModel() - No Engine Found " + ioe.getMessage());
                        }
                        if (sok.connected)
                        {
                                sok.write(tmpMap);
                                sok.read(tmpMap);
                                sok.close();
                        }
                        else // (!sok.connected)
                        {
                                tmpMap.put("sysResultStatus", "-1");
                                tmpMap.put("sysResultString", "Socket Connection failed");
                        }

                        if (tmpMap.containsKey("sysResultStatus") && tmpMap.get("sysResultStatus").equals("-1"))
                        {
                                System.err.println("Utils::verifyModel() - Cannot verify configured transaction model " +
                                                         tmpMap.get("sysResultString"));
                                validModel = true;
                        }
                        String modelName = null;
                        for (int i = 0; tmpMap.containsKey("sysModelName" + Integer.toString(i)); i++)
                        {
                                modelName = (String) tmpMap.get("sysModelName" + Integer.toString(i));
                                if (modelName.equals(model))
                                {
                                        validModel = true;
                                        System.err.println("Utils::verifyModel() - Configured model verified OK");
                                }
                        }
                        if (validModel == false)
                                throw new ThrowableString("Servlet::getVerifiedModel() - Execute model does not exist");
                }
                return(model);
        }
}


