
//-----------------------------------------------------------
//
//	Softbolts Ltd. (c) all rights reserved
//
//	Warning: This  program is  protected by copyright law 
//		 and   international  treaties.  Unauthorised
//		 reproduction or distribution of this program,
//		 or  any portion  or it, may result in severe
//		 civil  and  criminal  penalties, and will be
//		 prosecuted to  the  maximum extent possible.
//
//              Author: A. E. Dawson, 22 Apr 2003
//              mailto: anthony@softbolts.com
//
//		Last Revised: 22 Apr 2003
//
//-----------------------------------------------------------


package com.softbolts.monitors.TBMonitor;

import com.softbolts.lib.*;
import java.io.*;
import java.net.*;
import java.util.*;
import java.lang.Throwable;
import java.sql.*;

public class Servlet extends BaseServlet
{
 private static final int MIN_POLLTIME = 1;
 static final String keyPollingInterval = "Polling Interval";
 public PollerState pollerState = new PollerState();
 int pi = 0;

	public Servlet(String args[])
	{
		System.out.println("TBMonitor Started...");

		servletName = "TBMonitor";
		servletType = SvrTyp.monitor;
		servletState = RunStatusTyp.running;
		procMode = ProcModeTyp.single_threaded;

                AddConfigParam(keyPollingInterval, 5, "^[A-Za-z0-9_]*$", true);

	}

        //-----------------------------------------------------------
        public void service(NameValueTyp params) throws ThrowableString
        {
         String sysService = null;

                if (params.containsKey("sysService"))
                        sysService = (String) params.get("sysService");

                System.out.println("TBMonitor.Servlet::service() - " + sysService);
                if (params.containsKey("sysService") && ((String) params.get("sysService")).equals("Halt Servlet"))
                        stopPoller();
        }

	//-----------------------------------------------------------
	public void init(NameValueTyp params) throws ThrowableString
	{
		System.out.println("TBMonitor.Servlet::init()");
		if (pollerState.test() == true) stopPoller();

		pi = getPollingInterval();
		startPoller(pi);

		System.out.println("Done init()");
	}

	//-----------------------------------------------------------
	public void startPoller(int pi) throws ThrowableString
	{
											// fork off the poller thread
		System.err.println("Servlet::startPoller() - Attempting to start poller");
		System.err.println("Servlet::startPoller() - pi             : " + pi);


		if (pollerState.test() == false)
		{
			System.err.println("Servlet::init() - Starting poller");
			//MonitorPoller mp = new MonitorPoller(pollerState, pi, assignedName, this);
			MonitorPoller mp = new MonitorPoller(pollerState, pi, this);
			mp.start();
		}
		else	// should never happen as long as stopPoller has worked properly!
		{
			System.err.println("Servlet::init() - Poller already running");
			throw new ThrowableString("Servlet::init() - Poller already running");
		}
	}

        //-----------------------------------------------------------
        public void stopPoller()
        {
	 Thread cth = Thread.currentThread();

                if (pollerState.test() == true)
                {
								// tell the pooler to stop
								// once stopped the poller will 
								// indicate this by setting the 
								// pollerState flag
                        pollerState.reset();
			while (pollerState.test() == false)
			{
				try {
					cth.sleep(1000);	// thousands of one second
				} catch (InterruptedException ie) {
					System.err.println("TBDB.FilePoller::run() - InterruptedException on call to sleep()");
				}
			}
                        pollerState.reset();
                }
        }

        //-----------------------------------------------------------
        public int getPollingInterval() throws ThrowableString
        {
         int pi = MIN_POLLTIME;

                if (configParams.get(keyPollingInterval).value.equals("") == false)
                {
                        String intString = configParams.get(keyPollingInterval).value;
			pi = Integer.parseInt(intString);
                }

                if (pi < MIN_POLLTIME) pi = MIN_POLLTIME;

                return (pi);
        }

	//-----------------------------------------------------------
	public void finish(NameValueTyp params) throws ThrowableString
	{
		System.err.println("TBDB.Servlet::finish()");
		stopPoller();
		System.out.println("TBDB Connector Stopped...");
	}

	//-----------------------------------------------------------
	public void beginTransaction(NameValueTyp params) throws ThrowableString
	{
	}

	//-----------------------------------------------------------
	public void commitTransaction(NameValueTyp params) throws ThrowableString
	{
	}

	//-----------------------------------------------------------
	public void rollBackTransaction(NameValueTyp params) throws ThrowableString
	{
	}
}


